import UIKit
import Flutter

@UIApplicationMain
@objc class AppDelegate: FlutterAppDelegate, TJPlacementDelegate {
    
    var p = TJPlacement()
    var action = TJActionRequest()
    var timer = Timer()
    var channel: FlutterMethodChannel!
    
    public func requestDidSucceed(_ placement: TJPlacement!) {
        channel.invokeMethod("onRequestSuccess", arguments: nil)
    }
    public func requestDidFail(_ placement: TJPlacement!, error: Error!) {
        channel.invokeMethod("onRequestFailure", arguments: nil)
    }
    public func contentIsReady(_ placement: TJPlacement!) {
        channel.invokeMethod("onContentReady", arguments: 1)
    }
    public func contentDidAppear(_ placement: TJPlacement!) {
        channel.invokeMethod("onContentShow", arguments: nil)
    }
    public func contentDidDisappear(_ placement: TJPlacement!) {
        channel.invokeMethod("onContentDismiss", arguments: nil)
    }
    public func placement(_ placement: TJPlacement!, didRequestReward request: TJActionRequest!, itemId: String!, quantity: Int32) {
        channel.invokeMethod("onReward", arguments: quantity)
    }
    
    @objc func tjcConnectSuccess(notif: NSNotification) {
    NSLog("Tapjoy connect succeeded")
     channel.invokeMethod("onConnectSuccess", arguments: nil)
    }

    @objc func tjcConnectFail(notif: NSNotification) {
    NSLog("Tapjoy connect failed")
    }

    override func application(
        _ application: UIApplication,
        didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?
    ) -> Bool {
    NotificationCenter.default.addObserver(self, selector: #selector(AppDelegate.tjcConnectSuccess(notif:)), name: NSNotification.Name(rawValue: TJC_CONNECT_SUCCESS), object: nil)
    NotificationCenter.default.addObserver(self, selector: #selector(AppDelegate.tjcConnectFail(notif:)), name: NSNotification.Name(rawValue: TJC_CONNECT_FAILED), object: nil)
        let rootViewController : FlutterViewController = window?.rootViewController as! FlutterViewController
        channel = FlutterMethodChannel(name: "native", binaryMessenger: rootViewController as! FlutterBinaryMessenger)
        channel.setMethodCallHandler {(call: FlutterMethodCall, result: FlutterResult) -> Void in
            switch(call.method) {
            case "setDebugEnabled":
                 if let args = call.arguments as? Dictionary<String, Any>,
                let isDebug = args["isDebug"] as? Bool {
                    Tapjoy.setDebugEnabled(isDebug)
                } 
                break;
            case "setUserConsent":
                if let args = call.arguments as? Dictionary<String, Any>,
                let consent = args["consent"] as? String {
                  let privacyPolicy = TJPrivacyPolicy()
                  privacyPolicy.setUserConsent(consent)
                } 
                break;
            case "connect":
            if let args = call.arguments as? Dictionary<String, Any>,
                let tapjoyKey = args["api_token"] as? String {
                    Tapjoy.connect(tapjoyKey)
                }      
                break;
            case "getPlacement":
            if let args = call.arguments as? Dictionary<String, Any>,
                let placementName = args["placementName"] as? String {
                self.p = TJPlacement.placement(withName: placementName, delegate: self) as! TJPlacement
                }  
                self.p.requestContent()
                break;
            case "showContent":
                if self.p.isContentReady {
                    self.p.showContent(with: nil)
                } 
            break;
            case "isLoaded":
             if self.p.isContentReady {
                result(true)
                } else {
                result(false)
                }
            break;
            default: result(FlutterMethodNotImplemented)
            }
        }
        GeneratedPluginRegistrant.register(with: self)
        return super.application(application, didFinishLaunchingWithOptions: launchOptions)    
    }   
    
}
